/*--------------------------------------------------------------------
 * $Id: gencld.h,v 1.1 2002/07/31 14:07:29 pa2h Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/

#ifndef __gencld_h
#define __gencld_h

#define CLD_NONE      0
#define CLD_CASCADE1D 1
#define CLD_CASCADE2D 2
#define CLD_AUTOMATON 3
#define CLD_GAMMA     4


typedef struct {
  int periodic;
  int upward;

  double *cf;
  int    ncf;
} automaton_struct;


typedef struct {
  double fraction;
  double scale;
  int    step;
} cascade1D_struct;


typedef struct {
  double p1;
  double p2;
  double H;
  int    step;
} cascade2D_struct;


typedef struct {
  double gamma;
  int sort;
} gamma_struct;


/* structure to hold all input data */
typedef struct {
  int Nx;
  int Ny; 
  int Nz;

  double dx;
  double dy;

  char *infilename; 
  char *outfilename; 
  char *cloudfilename; 
  char *cffilename; 

  char type;
  automaton_struct automaton;
  cascade1D_struct cascade1D;
  cascade2D_struct cascade2D;
  gamma_struct     gamma;

  char images;

  double bottom;
  double top;
  double tau; 
  double ext; 

  int gspec;
  int wspec;
  int gwspec;

  double w;
  double g;
  double reff;

  int bottomlevel;
} input_struct;


int calc_new_profile (input_struct *in, double **zatm_out, int *nzatm_out);
int bounded1D (double tau, double f, double c, int step, double **result, int *n);
int bounded2D (double tau, double p1, double p2, double H, int step, 
	       double ***result, int *nx, int *ny);
void write_bounded1D_wcloud3D (input_struct *in, double *bound1D, 
				      double *zatm_out, int nzatm_out, 
				      FILE *f);

void write_bounded2D_wcloud3D (input_struct *in, double **bound2D, 
			       double *zatm_out, int nzatm_out, 
			       FILE *f);

int cloud_initialize (input_struct input, char ****cond, char ****newcond);
int summary(char ***cond, input_struct input,
	    int **cldsum, int **horsum, 
	    int *cldmax,  int *hormax,
	    double *cldavg,
	    int *ncld);
int remove_images();

int write_image(int **cldsum, int **horsum, 
		int cldmax, int hormax,
		int Nx, int Ny, int Nz,
		int counter);

int iterate (input_struct input, char ***cond, char ***newcond, char *succeed);

int cloud_reset (input_struct input, char ***cond);

void write_wcloud3D (input_struct input, char ***cond, 
		     double *zatm_out, int nzatm_out, 
		     double cldcover, int index, FILE *f);

int write_cloud (input_struct input, char ***cond, int **cldsum, double *zatm_out, int nzatm_out);

int rndgamma (double tau, double gamma, int Nx, int Ny, int sort, double ***result);

void write_gamma_wcloud3D (input_struct *in, double **gamma, 
			   double *zatm_out, int nzatm_out, 
			   FILE *f);

#endif
