/*--------------------------------------------------------------------
 * $Id: ckdfu.c,v 1.5 2002/12/09 23:29:19 pa2h Exp $
 * 
 * This file is part of libRadtran.
 * Copyright (c) 1997-2001 by Arve Kylling and Bernhard Mayer.
 * 
 * ######### Contact info: http://www.libradtran.org #########
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License   
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.        
 * 
 * This program is distributed in the hope that it will be useful, 
 * but WITHOUT ANY WARRANTY; without even the implied warranty of  
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the   
 * GNU General Public License for more details.                    
 * 
 * You should have received a copy of the GNU General Public License          
 * along with this program; if not, write to the Free Software                
 * Foundation, Inc., 59 Temple Place - Suite 330, 
 * Boston, MA 02111-1307, USA.
 *--------------------------------------------------------------------*/

#include <stdio.h>
#include <malloc.h>
#include "uvspec.h"
#include "../src_c/fortran_and_c.h"
#include "../src_f/f77-uscore.h"


int ckdfu (float umco2, float umch4, float umn2o, 
	   float umf11, float umf12, float umf22, 
	   float *rhoair, float *rhoh2o, float *rhoo3, 
	   float *pres, float *temp, float *z, int nlev, int h2o_cont,
	   ck_profile *ck)
{
  
  #include "fl_radparams.cinc"


  float *tmp_ck   = calloc (nv1x*mbx*mg, sizeof(float));
  float *tmp_delg = calloc (mbx*mg,      sizeof(float));

  float ***fck=NULL, **fdelg=NULL;
  int *kg = calloc (mbx, sizeof(int));
  int ig=0, ib=0, lu=0;

  void F77_FCN (ckdfuf) (float *umco2, float *umch4, float *umn2o, 
			 float *umf11, float *umf12, float *umf22, 
			 float *rhoair, float *rhoh2o, float *rhoo3, 
			 float *pres, float *temp, float *z, int *nlev, int *h2o_cont,
			 int *kg, float *tmp_delg, float *tmp_ck);
  
  F77_FCN  (ckdfuf) (&umco2, &umch4, &umn2o, 
		     &umf11, &umf12, &umf22,
		     rhoair, rhoh2o, rhoo3, 
		     pres, temp, z, &nlev, &h2o_cont, kg, tmp_delg, tmp_ck);
  
  fck   = fortran2c_3D_float_ary (mg, nvx, mbx, tmp_ck);
  fdelg = fortran2c_2D_float_ary (mg, mbx, tmp_delg);
  free (tmp_ck); free (tmp_delg); 


  /* Loop over bands */
  for (ib=0; ib<mbx; ib++) {
    
    /* copy data to final destination */
    ck[ib].ngauss = kg[ib];
    
    for (ig=0; ig<kg[ib]; ig++) {
      
      /* copy data to final destination */
      ck[ib].weight[ig] = fdelg[ig][ib];
      
      for (lu=0; lu<nlev-1; lu++)
	ck[ib].crs[lu][ig] = fck[ig][lu][ib];
    }
  }

  free (kg);
  return 0;
}



int ckdfucld (float *z, float *reff, float *lwc, int nlev,
	      float **tau, float **gg, float **ssa)
{
  
  #include "fl_radparams.cinc"


  float *tmp_tau   = calloc (nv1x*mbx, sizeof(float));
  float *tmp_ssa   = calloc (nv1x*mbx, sizeof(float));
  float *tmp_ww    = calloc (4*nv1x*mbx, sizeof(float));

  float **tmpc_tau=NULL, **tmpc_ssa=NULL, ***tmpc_ww=NULL;

  int ib=0, lu=0;

  void F77_FCN (ckdcldf) (float *z, float *reff, float *lwc, int *nlev,
			  float *tmp_tau, float *tmp_ww, float *tmp_ssa);
  
  F77_FCN  (ckdcldf) (z, reff, lwc, &nlev, tmp_tau, tmp_ww, tmp_ssa);

  tmpc_tau  = fortran2c_2D_float_ary (nvx, mbx, tmp_tau);
  tmpc_ssa  = fortran2c_2D_float_ary (nvx, mbx, tmp_ssa);
  tmpc_ww   = fortran2c_3D_float_ary (nvx, mbx, 4, tmp_ww);
  free (tmp_tau); free (tmp_ssa); free (tmp_ww); 

    
  /* Loop over bands */
  for (ib=0; ib<mbx; ib++) {

    /* copy data to final destination */
    for (lu=0; lu<nlev-1; lu++) { 
      tau[ib][lu] = tmpc_tau[lu][ib];
      ssa[ib][lu] = tmpc_ssa[lu][ib];
      gg [ib][lu] = tmpc_ww [lu][ib][0] / 3.0;  /* first moment of the phase function */
    }
  }
  
  return 0;
}



int ckdfuice (float *z, float *reff, float *lwc, int nlev,
	      float **tau, float **gg, float **ssa)
{
  
  #include "fl_radparams.cinc"

  float *tmp_tau   = calloc (nv1x*mbx, sizeof(float));
  float *tmp_ssa   = calloc (nv1x*mbx, sizeof(float));
  float *tmp_ww    = calloc (4*nv1x*mbx, sizeof(float));

  float **tmpc_tau=NULL, **tmpc_ssa=NULL, ***tmpc_ww=NULL;

  float *deff = calloc (nlev, sizeof(float));

  int ib=0, lu=0;

  void F77_FCN (ckdicef) (float *z, float *reff, float *lwc, int *nlev,
			  float *tmp_tau, float *tmp_ww, float *tmp_ssa);
  
  /* Fu et al. requires effective diameter */
  for (lu=0; lu<nlev; lu++)
    deff[lu] = 2.0 * reff[lu];

  F77_FCN  (ckdicef) (z, deff, lwc, &nlev, tmp_tau, tmp_ww, tmp_ssa);

  tmpc_tau  = fortran2c_2D_float_ary (nvx, mbx, tmp_tau);
  tmpc_ssa  = fortran2c_2D_float_ary (nvx, mbx, tmp_ssa);
  tmpc_ww   = fortran2c_3D_float_ary (nvx, mbx, 4, tmp_ww);
  free (tmp_tau); free (tmp_ssa); free (tmp_ww); 

    
  /* Loop over bands */
  for (ib=0; ib<mbx; ib++) {

    /* copy data to final destination */
    for (lu=0; lu<nlev-1; lu++) { 
      tau[ib][lu] = tmpc_tau[lu][ib];
      ssa[ib][lu] = tmpc_ssa[lu][ib];
      gg [ib][lu] = tmpc_ww [lu][ib][0] / 3.0;  /* first moment of the phase function */
    }
  }

  free(deff);

  return 0;
}




int ckdfuray (int ib, int ig, float u0, int nlev, float *tau)
{
  
  #include "fl_radparams.cinc"


  void F77_FCN (ckdrayf) (int *ib, int *ig, float *u0, int *nlev, float *tau);
  
  F77_FCN (ckdrayf) (&ib, &ig, &u0, &nlev, tau);

  return 0;
}




