c=======================================================================
c SBDART gaseous absorption module
c
c BM extracted code from SBDART version 1.50, September 12, 2002
c The code in depthscl.f required heavy changes to work with libRadtran;
c in particular, only the gaseous absorption things were kept while all 
c aerosol, cloud, and Rayleigh scattering stuff was removed.
c=======================================================================
      subroutine depthscl(kdist,nz,k,npass,wl,amu0,dtausc,
     &     dtls,dtcs,dtcv,dtlv,dtauk,wtk,
     &     dtaug,wt)
c
c routine:   depthscl
c
c purpose:   compute optical depths and other layer-by-layer info
c
c input:
c  nz       number of atmospheric layers
c  k        term in k-distribution expansion
c  npass    total number of k-distribution terms in expansion
c  wl       wavelength (microns) 
c  amu0     cosine of solar zenith
c  dtausc   scattering optical depth
c  dtcs     optical depth increment along slant path due to gaseous continua
c  dtls     o.d.i. along slant path due to gaseous lines (non beers)
c  dtcv     o.d.i. along vertical due to gaseous continua
c  dtlv     o.d.i. along vertical due to gaseous lines (non beers)
c  dtauk    o.d.i. of all three k-distibution terms
c  wtk      probabilities of k-distribution terms

c
c   output:
c  dtaug    optical depth increment along vertical
c  wt       probabilities for the k'th k-distribution pass 


      INCLUDE 'sbtaugas.inc'

      dimension dtausc(*), dtls(*),dtcs(*),dtcv(*),dtlv(*),
     &     dtauk(mxly,3), wtk(mxly,3)

      dimension dtaug(mxly)
c
c
      tauls=0.      
      taulv=0.
      tauc=0.
      tauk=0.
      tauscat=0.
      tgline=0.
      tgcont=0.
      taug1=0.
      taug2=0.
      taug3=0.       

      if(npass.gt.1) then
        wnorm=0.
        w1=0.
        w2=0.
        w3=0.
        do 10 j=1,nz
          w1=wtk(j,1)*dtlv(j)+w1
          w2=wtk(j,2)*dtlv(j)+w2
          w3=wtk(j,3)*dtlv(j)+w3
 10     continue
        wnorm=w1+w2+w3
        if(wnorm.eq.0) then
          w1=1.
          w2=0.
          w3=0.
        else
          w1=w1/wnorm
          w2=w2/wnorm
          w3=w3/wnorm
        endif
        if(k.eq.1) wt=w1
        if(k.eq.2) wt=w2
        if(k.eq.3) wt=w3
      else
        wt=1.0
      endif

      do 30 j=1,nz
        dtauc=dtcv(j)
        tauc=tauc+dtauc
        dtsct=dtausc(j)
        tauscat=tauscat+dtsct
        tauls=tauls+dtls(j)
        taulv=taulv+dtlv(j)
        ultrans=exp(-tauls)

c NOTE: gas continua absorption is grouped with other continuous
c       opacities,  this assumes dtcs=dtcv/amu


c normalize direct beam to slant path transmission by
c adjusting k-distributions optical depths.  
c kdist=1 no renormalization
c kdist=2 renormalize (clear sky situation)
c kdist=3 ramp correction factor to one as tauscat goes > 1

        if(npass.gt.1) then 
          taugx1=taug1+dtauk(j,1)
          taugx2=taug2+dtauk(j,2)
          taugx3=taug3+dtauk(j,3)

          if(tauls.lt.12. .and. kdist.ge.2) then
            call taucor(w1,w2,w3,taugx1,taugx2,taugx3,amu0,tauls,cf)
            if(kdist.eq.3.and.tauscat.gt.1) then
              ramp=exp(-tauscat)/exp(-1.)
              cf=cf*ramp+(1.-ramp)
            endif
            taugx1=taugx1*cf
            taugx2=taugx2*cf
            taugx3=taugx3*cf
          endif
          
          if(k.eq.1) dtgasl=(taugx1-taug1)
          if(k.eq.2) dtgasl=(taugx2-taug2)
          if(k.eq.3) dtgasl=(taugx3-taug3)
          dtgasl=max(dtgasl,0.)

          taug1=taugx1
          taug2=taugx2
          taug3=taugx3
          transk=w1*exp(-taug1/amu0)
     &          +w2*exp(-taug2/amu0)
     &          +w3*exp(-taug3/amu0)
        else
          if(taulv.gt..001) then
            afac=amu0*tauls/taulv
          else 
            afac=1.
          endif
          if(tauscat.gt.1.) then
            ramp=exp(-tauscat)/exp(-1.)
            afac=afac*ramp+(1.-ramp)
          endif
          dtgasl=afac*dtlv(j)
          transk=ultrans
        endif
        
        dtaug(j)=dtgasl+dtauc

 30   continue

c  compute optical depth correction to user angle slant paths

      return
      end

